#pragma rtGlobals=1		// Use modern global access method.

Menu "Particle Analysis"
"ROI tools", PA_ROI_tools()
"Single image", PA_singleImage()
"Image series", PA_imageSeries()
"PA to ROI Mask", PA_toRoiMask()
"Event series", PA_roiSeries()
"Event histogram",PA_RoiHistogram()
"Weibull Fit",PA_WeibullFit()
"Event correlation", PA_RoiCorrlation()
"ImageParticleStatistics", PA_ImageParticleStatistics()
"Neighbour Statistics", PA_neighborStatistics()
"Neighbour Alignment", PA_neighborAlignment()
"Voronoi tessellation", PA_VoronoiTesselation()
"Angle Statistics", PA_AngularStatistics()
end

Proc PA_ROI_tools()
	// recall data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr):
	// recall ROI tools
	WMCreateImageROIPanel();
	Variable/G threshold=10
	Variable/G hchannels=15
End

Function PA_singleImage()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String imageStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	NVAR layer = root:$(dfStr):layer
	//  input
	Variable/G threshold
	String objtypeStr="dark regions"
	variable objtype=2
	Prompt objtype,  "object type", popup "dark regions; bright regions;"
	variable mode=1
	Prompt mode, "threshold mode", popup "user level (0-255);halfway within ROI; V_avg of ROI; halfway within histogram;"
	variable thresh=threshold
	Prompt thresh, "countour threshold"
	variable upperthresh=255
	Prompt upperthresh, "image upper threshold"
	variable minArea=8
	Prompt minArea, "particle minimum area"
	doPrompt  "input particle analysis", objtype, mode, thresh, upperthresh, minArea
	
	// if stack choose layer
	variable nr = dimSize($imageStr,2)
	if (V_flag==0)
		wave M_imageplane
		if (exists("M_ROIMask")==1) // the ROI wave exists otherwise quit
			// bright or dark
			if (objtype ==2)
				objtypeStr="bright regions"
			endif
			// calc ROI area
			waveStats/Q M_ROIMask 
			variable areaTotROIMask=V_npnts-V_sum
			// calc threshold
			if (nr > 1)
				ImageTransform/P=(layer)/PTYP=0 getPlane $imageStr // get layer
			else			
				duplicate/O $imageStr M_ImagePlane
			endif
			
			// converts to gray
			M_imageplane[0,0]=0
			M_imageplane[0,1]=upperthresh
			M_imageplane=min(M_imageplane,upperthresh)
			M_imageplane=max(M_imageplane,0)
			ImageTransform convert2gray M_ImagePlane
			
			if (mode > 1)
				ImageStats/R=M_ROIMask M_Image2Gray
				if (mode == 2)
					thresh = (V_max+V_min)/2
				endif
				if (mode == 3)
					thresh = V_rms
				endif
				if (mode == 4)
					ImageHistogram/R=M_ROIMask M_Image2Gray
					thresh = 0.5*(PA_histogramThreshold(W_ImageHist,0.03)+PA_histogramThreshold(W_ImageHist,0.97))
				endif
			endif
			if (objtype==1)
				ImageThreshold/M=0/T=(thresh)/Q/R=M_ROIMask M_Image2Gray
			else
				ImageThreshold/I/M=0/T=(thresh)/Q/R=M_ROIMask M_Image2Gray
			endif
			ImageAnalyzeParticles /W/Q/M=3/A=(minArea)/R=M_ROIMask/F stats, root:$(dfStr):M_ImageThresh
			PA_boundaryToFront(winStr,"W_BoundaryY","W_BoundaryX") //AppendToGraph/T root:$(dfStr):W_BoundaryY vs root:$(dfStr):W_BoundaryX
			// calculate and print total area particles within ROI
			variable areaParticles=sum(W_ImageObjArea)
			variable errorAreaParticles=sum(W_ImageObjPerimeter)
			String textBoxMessage="Object: "+objtypeStr+";\r"+"Threshold = "+num2Str(thresh)+"; Minimum Area = "+num2Str(minArea)+";\r"+"Total area = "+num2Str(areaParticles/areaTotROIMask*100)+"  "+num2Str(errorAreaParticles/areaTotROIMask*50)+" %"
			TextBox/C/N=areatext/A=LB "\\F'Arial'\\Z10"+textBoxMessage
			print ReplaceString("\r", textBoxMessage, " ")
			threshold=thresh
		else
			print "please define ROI using ROI tools"
		endif
	endif
end

Function PA_imageSeries()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	// global variables
	NVAR layer = root:$(dfStr):layer
	SVAR selectedwaveStr=root:$(stackStr):selectedwaveStr
	wave wv=$selectedwaveStr
	wave M_imagePlane
	NVAR valdis = root:SIS:valdis
	variable/G threshold
	//
	variable timeflag=0
	String unitsStr
	// 
	variable nlayers = dimSize($stackStr,2)
	// if nlayer == 1 call singleimage
	if (nlayers < 2)
		PA_singleImage()
	endif
	//  input
	String objtypeStr="dark regions"
	variable objtype=2
	Prompt objtype,  "object type", popup "dark regions; bright regions;"
	variable mode=1
	Prompt mode, "threshold mode", popup "user level (0-255);% of ROI; V_avg of ROI; % of histogram; relative to histogram maximum;"
	variable perc=0.33
	Prompt perc, "percentage level during threshold"
	variable thresh=threshold
	Prompt thresh, "countour threshold"
	variable upperthresh=255
	Prompt upperthresh, "image upper threshold"
	variable minArea=8
	Prompt minArea, "particle minimum area"
	variable from=layer
	Prompt from, "from layer"
	variable to=nlayers-1
	Prompt to, "to layer"
	variable makeMovie=1
	Prompt makeMovie,  "make movie", popup "no; yes;"
	variable fps=5
	Prompt fps, "movie speed (frame/s): "
	doPrompt  "input particle analysis", objtype, mode, perc, thresh, upperthresh, minArea, from, to, makeMovie, fps
	if (V_flag==0)
		if (exists("M_ROIMask")==1) // the ROI wave exists otherwise quit
			if (makeMovie==2) // ask to save movie
				NewMovie /F=(fps)/L/O/Z/I as stackStr
				// define units
				if (stringmatch(selectedwaveStr,"pyro*")==1)
					unitsStr=" C"
				endif
				if (stringmatch(selectedwaveStr,"temp*")==1)
					unitsStr=" C"
				endif
				if (stringmatch(selectedwaveStr,"PMCH*")==1)
					unitsStr=" mbar"
				endif
				if (stringmatch(selectedwaveStr,"time*")==1)
					unitsStr=" s"
					timeflag=1
				endif
			endif
			// initialize waves
			String areaStr=dfStr+"_AR"
			String perimeterStr=dfStr+"_PE"
			String areaerrorStr=dfStr+"_ARE"
			String nparticlesStr=dfStr+"_NP"
			make/n=(nlayers)/O  $areaStr=NaN,   $areaerrorStr=NaN,  $perimeterStr=NaN, $nparticlesStr=NaN
			wave aw=$areaStr
			wave aew=$areaerrorStr
			wave pw=$perimeterStr
			wave npw=$nparticlesStr
			// initialize other variables
			if (objtype ==2)
				objtypeStr="bright regions"
			endif
			// calc ROI area
			waveStats/Q M_ROIMask 
			variable areaTotROIMask=V_npnts-V_sum
			// calc threshold for the selected layer
			ImageTransform/P=(layer)/PTYP=0 getPlane $stackStr // puts layer into M_ImagePlane
			M_imageplane[0,0]=0
			M_imageplane[0,1]=upperthresh
			M_imageplane=min(M_imageplane,upperthresh)
			M_imageplane=max(M_imageplane,0)
			ImageTransform convert2gray M_ImagePlane // creates binary image and store result in M_Image2grey
			variable hmax=NaN
			ImageStats/R=M_ROIMask M_Image2Gray
			print "average value within ROI", V_avg
			if (mode > 1)
				if (mode == 2)
					thresh = perc*(V_max+V_min)
				endif
				if (mode == 3)
					thresh = V_rms
				endif
				if ((mode == 4) || (mode == 5))
					ImageHistogram/R=M_ROIMask M_Image2Gray
					thresh = perc*(PA_histogramThreshold(W_ImageHist,0.03)+PA_histogramThreshold(W_ImageHist,0.97))
					wavestats/Q W_ImageHist
					hmax=V_maxloc
				endif
			endif
			// loop
			variable i=from
			variable thresh_i
			String textBoxMessage=""
			variable unitstep= 1 
			if (to<from)
				unitstep=-1
			endif
			//print from,to
			do
				// sho layer
				layer=i
				modifyImage $(stackStr) plane=(i)
				valdis=wv[i]
				ValDisplay vd,value=#num2Str(valdis)
				// layer operations
				ImageTransform/P=(i)/PTYP=0 getPlane $stackStr // get layer
				M_imageplane[0,0]=0
				M_imageplane[0,1]=upperthresh
				M_imageplane=min(M_imageplane,upperthresh)
				M_imageplane=max(M_imageplane,0)
				ImageTransform convert2gray M_ImagePlane				
				// threshold				
				if (mode == 5)
					ImageHistogram/R=M_ROIMask M_Image2Gray
					Smooth 4, W_ImageHist
					WaveStats/Q W_ImageHist
					thresh_i = thresh*V_maxloc/hmax
				else
					thresh_i=thresh
				endif
					
				// calculate threshold image for bright or dark particles
				if (objtype==1)
					ImageThreshold/M=0/T=(thresh_i)/Q/R=M_ROIMask M_Image2Gray
				else
					ImageThreshold/I/M=0/T=(thresh_i)/Q/R=M_ROIMask M_Image2Gray
				endif 
				ImageAnalyzeParticles /W/Q/M=3/A=(minArea)/R=M_ROIMask/F stats, root:$(dfStr):M_ImageThresh

				// update if i==from
				if (i==from)
					PA_boundaryToFront(winStr,"W_BoundaryY","W_BoundaryX") 
				endif
				// calculate and print total area particles within ROI
				npw[i]=V_NumParticles
				aw[i]=sum(W_ImageObjArea)/areaTotROIMask*100
				aew[i]=sum(W_ImageObjPerimeter)/areaTotROIMask*50
				pw[i]=sum(W_ImageObjPerimeter)/areaTotROIMask*100
				// print texbox on stack 
				if (makeMovie==1)
					textBoxMessage="i = "+num2Str(i)+"; Object: "+objtypeStr+";\r"+"Threshold = "+num2Str(thresh)+"; Minimum Area = "+num2Str(minArea)+";\r"+"Total area = "+num2Str(aw[i])+"  "+num2Str(pw[i])+" %"
					TextBox/C/N=areatext/A=LB "\\F'Arial'\\Z10"+textBoxMessage
					//print ReplaceString("\r", textBoxMessage, " ")
				else
					if (timeflag==1)
						sprintf textbox2Str "%*.*f" 4, 2, wv[i]/1000
					else
						sprintf textboxMessage "%*.*f" 4, 2, wv[i]
					endif
					TextBox/C/N=areatext/A=LB textboxMessage+unitsStr
				endif
				// update
				doUpdate
				// movie
				if (makeMovie==2) //  save movie frame
					AddMovieFrame
				endif
				// loop
				i+=unitstep
			while ((i<=abs(to)) && (i>-1))
			if (makeMovie==2) // close movie
				closeMovie
			endif
			// restore layer
			// sho layer
			layer=0
			modifyImage $(stackStr) plane=(0)
			valdis=wv[0]
			ValDisplay vd,value=#num2Str(valdis)
			//
			removeFromGraph W_BoundaryY
			TextBox/K/N=areatext
			doUpdate
			// display results if ...
			variable axis=2
			Prompt axis, "x axis", popup "image number;time;temperature (TC);temperature (PY);other;"
			doPrompt "axes label" axis
			if (V_flag==0)	
				String xStr
				String graphStr="PA"+dfStr
				String paxStr="PA"+dfStr+"_P"
				// image number
				if (axis == 1)
					Prompt xStr, "enter wave",popup WaveList("*NUM*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Image number"
					endif
				endif
				// time
				if (axis == 2)
					Prompt xStr, "enter wave",popup WaveList("ti*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						wave paxw=$paxStr
						paxw/=1000 
						//SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						//SIS_xywaveToRightAxis(graphStr,nparticlesStr,paxStr)
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Time (s)"
					endif
				endif
				// temperature (TC)
				if (axis == 3)
					Prompt xStr, "enter wave",popup WaveList("te*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Temperature (C)"
					endif
				endif
				// temperature (Pyrometer)
				if (axis == 4)
					Prompt xStr, "enter wave",popup WaveList("py*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Temperature (C)"
					endif
				endif				
				// other
				if (axis == 5)
					Prompt xStr, "enter wave",popup WaveList("*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Other"
					endif
				endif
				ModifyGraph fSize=12
				Label areaaxis "\\F'Arial'\\Z14Area fraction (%)"
				Label particleaxis "\\F'Arial'\\Z14Number of particles"
				ModifyGraph mirror=1,axisEnab(areaaxis)={0.52,1},axisEnab(particleaxis)={0,0.48};DelayUpdate
				ModifyGraph freePos(areaaxis)=0,freePos(particleaxis)=0
				ModifyGraph lblPos(areaaxis)=60,lblPos(particleaxis)=60,lblLatPos=0
				ModifyGraph rgb($areaStr)=(65280,0,0);DelayUpdate
				ModifyGraph rgb($nparticlesStr)=(256,0,63232)
				ErrorBars $areaStr Y,wave=($areaerrorStr,$areaerrorStr)
			endif
		else
			print "please define ROI using ROI tools"
		endif
	endif
end

Function PA_toRoiMask()
	wave M_particle
	duplicate/o M_particle M_RoiMask
	M_RoiMask-=16
End

Function PA_roiSeries()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	// global variables
	wave W_xmin = root:$(dfStr):W_xmin
	wave W_xmax = root:$(dfStr):W_xmax
	wave W_ymin = root:$(dfStr):W_ymin
	wave W_ymax = root:$(dfStr):W_ymax
	variable npa = dimSize(W_xmin,0)	
	print "there are:"+num2str(npa)+" particles"
	// make roimask
	wave ws=$stackStr	
	variable np = dimSize(ws,0)	
	variable nq = dimSize(ws,1)
	variable nr = dimSize(ws,2)
	variable ir=0
	print "p.q:"+num2str(np)+","+num2str(nq)	
	make/n=(np,nq)/o/b/u PA_RoiMask=1
	make/n=(nr)/o wROI,dROI,wx=p
	make/n=(nr*10)/o w_eventTime,w_eventPosX,w_eventPosY,w_eventIndex
	variable ievent=0
//newImage/S=2 PA_RoiMask
//display wroi
	variable ipa=0
	variable thresh = 0
	variable j=0
	// input
	variable nsdev=3
	Prompt nsdev, "threshold", popup "1 sdev; 2 sdev; 3 sdev; 4 sdev; 5 sdev;"
	variable nrepeatedevents=5
	Prompt nrepeatedevents, "number of events per particle", popup "1; 2; 3; 4; 5;"
	variable axis=2
	Prompt axis, "x axis", popup "image number;time;temperature (TC);temperature (PY);other;"
	doPrompt "axes label" nsdev,nrepeatedevents,axis
	if (V_flag==0)	
		if (axis == 2)
			String xStr
			Prompt xStr, "enter wave",popup WaveList("ti*",";","")
			doPrompt "bottom axis input wave" xStr
			if (V_flag==0)	
				duplicate/o $xStr wx
				wx/=1000 
			endif
		endif	
	endif
	do
		// calculates the mask
		PA_RoiMask=1
		PA_RoiMask[W_xmin[ipa],W_xmax[ipa]][W_ymin[ipa],W_ymax[ipa]]=0
		// extract the beam
		ir=0
		do
			ImageStats/P=(ir)/R=PA_ROIMask ws 
			wROI[ir]=V_avg
			ir+=1
		while(ir<nr)
		// process the beam
		Smooth 2, wROI	
		Differentiate wROI /D =dROI
		wavestats/Q dROI
		doUpdate
		thresh = V_avg - nsdev*V_sdev
		j=0
		do 
			wavestats/Q dROI
			if (V_min < thresh)
				print "found event @: ",wx[V_minloc],V_minloc,V_min,thresh,(W_xmin[ipa]+W_xmax[ipa])/2,(W_ymin[ipa]+W_ymax[ipa])/2
				w_eventTime[ievent]=wx[V_minloc]
				w_eventPosX[ievent]=(W_xmin[ipa]+W_xmax[ipa])/2
				w_eventPosY[ievent]=(W_ymin[ipa]+W_ymax[ipa])/2
				w_eventIndex[ievent]=ievent
				dROI[V_minloc-3,V_minloc+3]=V_avg
				ievent+=1
			endif
			j+=1
		while (j < nrepeatedevents-1)
		ipa+=1
	while (ipa < npa)
	redimension/n=(ievent) w_eventTime, w_eventPosX, w_eventPosY, w_eventIndex
end

Function PA_RoiHistogram()
	variable channels=25
	Prompt channels, "number of channels"
	doPrompt  "input particle analysis ROI histogram", channels
	Make/N=(channels)/O w_eventTime_Hist
	Make/N=(channels)/O w_eventTime_Hist
	Histogram/B=1 w_eventTime,w_eventTime_Hist
	Display w_eventTime_Hist
	Integrate w_eventTime_Hist
	w_eventTime_Hist/=DimDelta(w_eventTime_Hist, 0 )
	Label left "\\F'Arial'\\Z16Total number of events ";DelayUpdate
	Label bottom "\\F'Arial'\\Z16Time (s)"
	ModifyGraph marker=8,mode(w_eventTime_Hist)=3,mode(fit_w_eventTime_Hist)=0
	//w_eventTime_Hist*=-1
	//Wavestats/Q w_eventTime_Hist
	//w_eventTime_Hist-=V_min
end


Function PA_RoiCorrlation()
	wave w_eventTime, w_eventPosX, w_eventPosY, w_eventIndex
	variable n2 = dimSize(w_eventTime,0)
	variable nsq= n2^2
	make/n=(nsq)/o distanceR, deltaT
	print n2
	variable k=0
	variable i,j
	for (i=0;i<n2;i+=1)
		for (j=0;j<n2;j+=1)
			distanceR[k]=sqrt( (w_eventPosX[i]-w_eventPosX[j])^2 + (w_eventPosY[i]-w_eventPosY[j])^2 )
			deltaT[k]= abs (w_eventTime[i]-w_eventTime[j]) 
			//print i,j,k
			k+=1				
		endfor
	endfor
	display distanceR vs deltaT
end

Function PA_WeibullFit()
	Wavestats/Q w_eventTime_Hist
	Make/n=4/o W_coef={V_max,V_maxloc/2,1,-60}
	Make/O/T/N=2 T_Constraints
	T_Constraints[0] = {"K2 > 0","K2 < 2"}
	FuncFit/H="1000"/NTHR=0 Weibull W_coef  w_eventTime_Hist /D /C=T_Constraints 
	ModifyGraph mode(fit_w_eventTime_Hist)=0,rgb(fit_w_eventTime_Hist)=(0,0,0)
end

Function PA_histogramThreshold(hw,thresh)
	wave hw
	variable thresh
	variable s=0,i=0
	variable tot= sum(hw)
	do
		s += hw[i]
		i+=1
	while( (s/tot) < thresh )
	return i
end

Function PA_boundaryToFront(graphNameStr,waveNameStr,wavexNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/T $waveNameStr vs $wavexNameStr
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph/T $waveNameStr vs $wavexNameStr
		endif
	endif
end

function PA_ImageParticleStatistics()
	// datafolder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// calculate center
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	duplicate/o W_SpotX baricenter_x
	duplicate/o W_SpotY baricenter_y
	wave baricenter_x
	baricenter_x = (W_xmin+W_xmax)/2
	wave baricenter_y
	 baricenter_y = (W_ymin+W_ymax)/2
	 wave angles, distances
	 duplicate/o W_SpotY, nndistw,tmpw, sortw	
	// calc nearest neighbour distances 
	variable i =0
	variable ipartx,iparty
	variable np = numpnts(baricenter_x)
	do
		ipartx = baricenter_x[i]
	 	iparty = baricenter_y[i]
	    	tmpw = sqrt((baricenter_x-ipartx)^2+(baricenter_y-iparty)^2)
	    	duplicate/o tmpw,sortw
	    	sort sortw,sortw
	      nndistw[i]=mean(sortw,1,7)
	      i+=1
	while (i < np)
	Wavestats/Q nndistw
	variable/G nnd = V_avg
	// Create histogram of radius/area
	wave W_imageObjArea, W_imageObjFOV
	variable vmed
	variable/G hchannels
	// input
	variable FoV=6
	Prompt FoV,"field of view (um)"
	variable binning=2
	Prompt binning, "binning", popup "1x1;2x2;4x4;8x8"
	variable mode=1
	Prompt mode, "mode", popup "radius;area"
	variable channels=hchannels
	Prompt channels, "histogram channels"
	variable fitType=1
	Prompt fitType, "fit function", popup "Gauss;LogNormal;LSW attachment limit;LSW diffusion limit;"
	doPrompt "input" FoV,binning, mode, channels, fitType
	if (V_flag==0)
		binning=2^(binning-1)
		hchannels=channels
		//print binning
		String graphNameStr 
		String histoStr
		duplicate/o W_ImageObjArea W_ImageObjFOV
		variable px2nm = 1000*FoV/(870/binning)
		if (mode == 1)
			W_ImageObjFOV=sqrt(W_ImageObjArea/Pi)*px2nm
			waveStats/Q W_imageObjFOV
			vmed = V_avg
			print "average radius", Vmed ,"(nm)"
			graphNameStr = "HR"+dfStr
			histoStr = dfStr+"_HR"
		elseif (mode == 2)
			W_ImageObjFOV=W_ImageObjArea*px2nm^2
			waveStats W_imageObjFOV
			vmed = V_avg
			print "average are", Vmed ,"(nm^2)", V_sdev
			graphNameStr = "HA"+dfStr
			histoStr = dfStr+"_HA"
		endif
		Make/N=1/O $histoStr
		Histogram/B={0,Vmed/(channels/2),channels} W_ImageObjFOV, $histoStr
		SIS_waveToFront(graphNameStr,histoStr)
		ModifyGraph fSize=12;DelayUpdate
		Label left "\\F'Arial'\\Z16Occurrence";DelayUpdate
		ModifyGraph offset($histoStr)={-Vmed/10,0}
		if (mode == 1)
			Label bottom "\\F'Arial'\\Z16Radius (nm)"
		elseif (mode == 2)
			Label bottom "\\F'Arial'\\Z16Area  (nm\\S2\\M\\F'Arial'\\Z16)"
		endif
		ModifyGraph mode($histoStr)=5
		print "nearest neighbour distance: ",nnd*px2nm, "(nm)" 
	  endif
	  killwaves/z nndistw,tmpw,sortw
	  if (fitType == 1)
	  	CurveFit/NTHR=0 gauss $histoStr /D 
	  elseif (fitType == 2) 
	  	make/n=4/O w_coef=0
	  	w_coef[0]=0
	  	waveStats/Q  $histoStr
	  	w_coef[1]=V_max
	  	w_coef[2]=V_maxloc*0.9
	  	w_coef[3]=0.5
	  	CurveFit/G/H="1000"/NTHR=0 LogNormal kwCWave=W_coef, $histoStr[1,channels-1] /D
	   elseif (fitType == 3) 
	   	make/n=2/O w_coef=0
	   	waveStats/Q  $histoStr
	   	w_coef[0]=V_max^2
	   	w_coef[1]=V_maxloc*0.9
	   	FuncFit/NTHR=0 LSW_AL w_coef  $histoStr /D 
	  elseif (fitType == 4) 
	   	make/n=2/O w_coef=0
	   	waveStats/Q  $histoStr
	   	w_coef[0]=V_max^2
	   	w_coef[1]=V_maxloc*0.9
	   	FuncFit/NTHR=0 LSW_DL w_coef  $histoStr /D 
	  endif 
end

function PA_particlePerAngle()
	// relevant waves
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	// define particle baricenter
	duplicate/o W_SpotX pb_x
	duplicate/o W_SpotY pb_y
	wave pb_x
	pb_x = (W_xmin+W_xmax)/2
	wave pb_y
	pb_y = (W_ymin+W_ymax)/2
	// centre of ROI
	variable xc=219
	variable yc=232
	variable thres=5
	// q = 232+p*tg(apha)
	wave W_ImageObjArea
	wave ang,angdis
	make/N=179/O ang,angdis
	// loop
	variable i =-89
	variable j=0
	variable angle
	variable mtg
	variable ql
	do
	 	angle = i*Pi/180
	 	mtg = tan(angle)
	 	ang[j]=i
	 	// debug
	 	//print i,angle,mtg
	 	// define line
	 	duplicate/o pb_x tmp
	 	ql = yc-mtg*xc
	 	// distances
	 	tmp = abs(pb_y - (mtg*pb_x+ql))/sqrt(1+mtg^2)
	 	// distances less than particle radius
	 	tmp = tmp <= sqrt(W_ImageObjArea)/Pi
	 	// 
	 	angdis[j]=sum(tmp)
	 	doUpdate
	 	i+=1
	 	j+=1
	 while (i< 90)
	 display angdis vs ang
end

function PA_AngularStatistics()
	// datafolder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// relevant waves
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	// define particle baricenter
	duplicate/o W_SpotX pb_x
	duplicate/o W_SpotY pb_y
	wave pb_x
	pb_x = (W_xmin+W_xmax)/2
	wave pb_y
	pb_y = (W_ymin+W_ymax)/2
	// relevant waves
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	// define particle baricenter
	duplicate/o W_SpotX pb_x
	duplicate/o W_SpotY pb_y
	wave pb_x
	pb_x = (W_xmin+W_xmax)/2
	wave pb_y
	pb_y = (W_ymin+W_ymax)/2
	//
	wave angle,angler,nmatches
	make/N=179/O angle=-89+p
	make/N=179/O angler=angle*Pi/180
	make/N=179/O nmatches=0
	wave W_ImageObjArea
	// loop
	variable maxj = 0
	variable j=0
	variable i=0
	variable np = numpnts(pb_x)
	duplicate/o pb_x tmp,tmp_ql
	variable pxc,pyc
	variable mtg //thres=50
	do
		pxc=pb_x[i]
		pyc=pb_y[i]
		for(j=0;j<179;j+=1)
			mtg=tan(angler[j])
			tmp_ql = pyc-mtg*pxc
			// distanza punto retta
			tmp = abs(pb_y - (mtg*pb_x+tmp_ql))/sqrt(1+mtg^2)
			tmp = ( tmp <= sqrt(W_ImageObjArea/Pi) )
			nmatches[j]=nmatches[j]+sum(tmp)-1
			// doUpdate
		endfor
		doUpdate
		i+=1
	while (i < np)
	nmatches/=(np)
	String graphNameStr="HN"+dfStr
	SIS_graphToFrontRight(graphNameStr,"nmatches","angle","right")
end

function PA_neighborStatistics()
	// datafolder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// relevant waves
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	// input
	variable FoV=6
	Prompt FoV,"field of view (um)"
	variable binning=2
	Prompt binning, "binning", popup "1x1;2x2;4x4;8x8"
	variable nneighbors=1
	Prompt nneighbors, "number of neighbors",popup "1;2;3;4;5;6;"
	doPrompt "rescale px nm" FoV,binning,nneighbors
	if (V_flag==0)
		binning=2^(binning-1)
		// define particle baricenter
		duplicate/o W_SpotX pb_x
		duplicate/o W_SpotY pb_y
		wave pb_x
		pb_x = (W_xmin+W_xmax)/2
		wave pb_y
		pb_y = (W_ymin+W_ymax)/2
		variable np= numpnts(pb_x)
		// waves
		wave wdij,wareaij,wangleij
		String dijStr = dfStr+"_Dij"
		String areaijStr = dfStr+"_Aij"
		String angleijStr = dfStr+"_NGij"
		make/N=(np^2/2)/O $dijStr,$areaijStr,$angleijStr
		wave wdij = $dijStr
		wave wareaij = $areaijStr
		wave wangleij = $angleijStr
		wave W_ImageObjArea
		wareaij = 0
		// main loop
		variable maxj = 0
		variable j=0
		variable i=0
		variable k=0
		duplicate/o pb_x wtmp
		variable pxi,pyi
		do
			// i-th particle
			pxi=pb_x[i]
			pyi=pb_y[i]
			wtmp = sqrt((pb_x-pxi)^2+(pb_y-pyi)^2)
			wtmp[i]=1E7
			// loop on neighbors
			k=0
			do
				waveStats/Q/R=(i,np-1) wtmp
				if (V_minloc > i)
					wdij[j]=wtmp[V_minloc]
					//wareaij[j]=W_ImageObjArea[V_minloc]
					//wareaij[j]=W_ImageObjArea[i]
					wareaij[j]=0.5*(W_ImageObjArea[V_minloc]+W_ImageObjArea[i])
					wangleij[j]=atan((pyi-pb_y[V_minloc])/(pxi-pb_x[V_minloc]))*180/Pi
					j+=1
				endif
				wtmp[V_minloc]=1E7
				k+=1
			while (k < nneighbors)
			i+=1
		while (i < np)
		redimension/n=(j) wareaij,wdij,wangleij
		String graphNameStr="AD"+dfStr
		SIS_xywaveToLeftAxis(graphNameStr,areaijStr,dijStr)
		Label left "\\F'Arial'\\Z16Area (\\u \\F'Arial'\\Z16nm\\S2\\M\\F'Arial'\\Z16)"
		Label bottom "\\F'Arial'\\Z16Distance (nm)"
		variable px2nm = 1000*FoV/(870/binning)
		wdij*=px2nm
		wareaij*=px2nm^2
		ModifyGraph mode=3,marker=19,rgb=(0,15872,65280)
		ModifyGraph fSize=12
		waveStats/Q $dijStr
		SetAxis bottom 0,V_max*1.1
		waveStats/Q $areaijStr
		SetAxis left 0,V_max*1.1
		doupdate
		String histoStr = dfStr+"_HI"
		String histogramNameStr="HI"+dfStr
		Make/N=1/O $histoStr
		Histogram/B={-90,1,180} $angleijStr, $histoStr
		SIS_waveToFront(histogramNameStr,histoStr)
		ModifyGraph fSize=12;DelayUpdate
       	Label left "\\F'Arial'\\Z16Occurrence";DelayUpdate
       	Label bottom "\\F'Arial'\\Z16Angle (deg)";
       	ModifyGraph mode=5
	endif
end

function PA_neighborAlignment()
	// datafolder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// relevant waves
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	// input
	variable FoV=6
	Prompt FoV,"field of view (um)"
	variable binning=2
	Prompt binning, "binning", popup "1x1;2x2;4x4;8x8"
	variable nneighbors=1
	Prompt nneighbors, "number of neighbors",popup "1;2;3;4;5;6;"
	doPrompt "rescale px nm" FoV,binning,nneighbors
	if (V_flag==0)
		binning=2^(binning-1)
		// define particle baricenter
		duplicate/o W_SpotX pb_x
		duplicate/o W_SpotY pb_y
		wave pb_x
		pb_x = (W_xmin+W_xmax)/2
		wave pb_y
		pb_y = (W_ymin+W_ymax)/2
		variable np= numpnts(pb_x)
		// waves
		wave wdij,wareaij,wangleij
		String dijStr = dfStr+"_Dij"
		String areaijStr = dfStr+"_Aij"
		String angleijStr = dfStr+"_NGij"
		String anglejkStr = dfStr+"_NGjk"
		make/N=(np^2/2)/O $dijStr,$areaijStr,$angleijStr,$anglejkStr
		wave wdij = $dijStr
		wave wareaij = $areaijStr
		wave wangleij = $angleijStr
		wave wanglejk = $anglejkStr
		wave W_ImageObjArea
		wareaij = 0
		// main loop
		variable maxj = 0
		variable j=0
		variable i=0
		variable k=0
		variable m=0
		variable n=0
		duplicate/o pb_x wtmp,wtmp2
		variable pxi,pyi,pxj,pyj
		variable dij,djk,angleij,anglejk,areaij
		do
			// i-th particle
			pxi=pb_x[i]
			pyi=pb_y[i]
			wtmp = sqrt((pb_x-pxi)^2+(pb_y-pyi)^2)
			wtmp[i]=1E7
			// loop on nn
			n=0
			do
				// only the closest is considered
				waveStats/Q/R=(i,np-1) wtmp
				j = V_minloc
				if (j > i)
					// calc dist i-j, average area, angle i-j
					dij=wtmp[j]
					areaij=0.5*(W_ImageObjArea[j]+W_ImageObjArea[i])
					angleij=atan((pyi-pb_y[j])/(pxi-pb_x[j]))*180/Pi
					// consider now m and find its nerest neighbor
					pxj=pb_x[j]
					pyj=pb_y[j]	
					wtmp2 = sqrt((pb_x-pxj)^2+(pb_y-pyj)^2)
					//wtmp[i]=1E7
					wtmp2[j]=1E7
					waveStats/Q/R=(i,np-1) wtmp2
					k=V_minloc
					if (k > j )
						// get the closeset
						djk=wtmp2[k]
						wdij[m]=dij
						wareaij[m]=areaij
						anglejk=atan((pyj-pb_y[k])/(pxj-pb_x[k]))*180/Pi
						wanglejk[m]=anglejk-angleij
						// debug
						//print i,j,k, dij,djk,angleij,anglejk
						m+=1
					endif
				endif
				wtmp[j]=1E7
				n+=1
			while (n < nneighbors)
			i+=1
		while (i < np)
		print m
		redimension/n=(m) wareaij,wdij,wangleij,wanglejk
		String graphNameStr="AD"+dfStr
		SIS_xywaveToLeftAxis(graphNameStr,areaijStr,dijStr)
		Label left "\\F'Arial'\\Z16Area (\\u \\F'Arial'\\Z16nm\\S2\\M\\F'Arial'\\Z16)"
		Label bottom "\\F'Arial'\\Z16Distance (nm)"
		variable px2nm = 1000*FoV/(870/binning)
		wdij*=px2nm
		wareaij*=px2nm^2
		ModifyGraph mode=3,marker=19,rgb=(0,15872,65280)
		ModifyGraph fSize=12
		waveStats/Q $dijStr
		SetAxis bottom 0,V_max*1.1
		waveStats/Q $areaijStr
		SetAxis left 0,V_max*1.1
		doupdate
		String histoStr = dfStr+"_HINN"
		String histogramNameStr="HINN"+dfStr
		Make/N=1/O $histoStr
		Histogram/B={-90,1,180} $anglejkStr, $histoStr
		SIS_waveToFront(histogramNameStr,histoStr)
		ModifyGraph fSize=12;DelayUpdate
       	Label left "\\F'Arial'\\Z16Occurrence";DelayUpdate
       	Label bottom "\\F'Arial'\\Z16Relative angle (deg)";
       	ModifyGraph mode=5
	endif
end

function PA_VoronoiTesselation()
	// datafolder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	String graphNameStr="SI"+dfStr
	SIS_imageToFront(graphNameStr,stackStr) 
	Variable/G threshold 
	// relevant waves
	wave W_SpotX, W_xmin, W_xmax
	wave W_SpotY, W_ymin, W_ymax
	wave  W_boundaryX
	wave  W_boundaryY
	// define particle baricenter
	duplicate/o W_SpotX pb_x
	duplicate/o W_SpotY pb_y
	wave pb_x
	pb_x = (W_xmin+W_xmax)/2
	wave pb_y
	pb_y = (W_ymin+W_ymax)/2
	variable np= numpnts(pb_x)
	// make triplet
	String partStr=dfStr+"_PART"
	Make/O/N=(np,3) $partStr
	wave wpart=$partStr
	wpart[][0]=pb_x[p]
	wpart[][1] =pb_y[p]
	wpart[][2]=threshold
	// Voronoi
	ImageTransform/R=M_ROIMask voronoi wpart
	PA_appendMultiWaveToGraphTop(graphNameStr,partStr)
	//AppendToGraph/T wpart[][1] vs wpart[][0]
	ModifyGraph mode($partStr)=3,marker($partStr)=19,msize($partStr)=1,rgb($partStr)=(0,0,65535)
	//Appendtograph/T  M_VoronoiEdges[][1] vs M_VoronoiEdges[][0]
	PA_appendMultiWaveToGraphTop(graphNameStr,"M_VoronoiEdges")
	SetAxis top 0,dimSize($stackStr,0)
	SetAxis left 0,dimSize($stackStr,1)
End 

Function PA_appendMultiWaveToGraphTop(graphNameStr,waveNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	String leftAxisName
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	wave w=$waveNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/T w[][1] vs w[][0]
		//Legend/C/N=text0/A=LT
		//ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		//ModifyGraph width=240,height={Aspect,1}
		//ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph/T w[][1] vs w[][0]
			//ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

function Weibull(w,x) : FitFunc
	Wave w
	Variable x
       return w[0]*(1-(exp(-((x-w[3])/w[1])^w[2])))
End

function StretchedExp(w,x) : FitFunc
	Wave w
	Variable x
       return w[0]*(exp(-((x)/w[1])^w[2]))+w[3]
End

// Hausser, Voigt Phys Rev B 72, 035437 (2005)
// attachment limit
// se equation 14 for justification
function LSW_AL(w,x) : FitFunc
	wave w
	variable x
	variable rho=x/w[1]
	//variable //t1=rho^2
	//variable //t2=(3/(3+rho))^7/3
	//variable //t3=(1.5/1.5-rho)^11/3
	//variable //t4=exp(-rho/(1.5-rho))
	if (rho <= 2)
		variable t1=8*rho
		variable t2=1/(2-rho)^4
		variable t3=1
		variable t4=exp(-4/(2-rho))
		return w[0]*t1*t2*t3*t4
	else
		return 0
	endif
End

// detachment limit
function LSW_DL(w,x) : FitFunc
	wave w
	variable x
	variable rho=x/w[1]
	//variable //t1=rho^2
	//variable //t2=(3/(3+rho))^7/3
	//variable //t3=(1.5/1.5-rho)^11/3
	//variable //t4=exp(-rho/(1.5-rho))
	if (rho <= 1.5)
		variable t1=rho^2
		variable t2=(rho+3)^(-17/9)
		variable t3=(3-2*rho)^(-28/9)
		variable t4=exp(-2/(3-2*rho))
		return w[0]*t1*t2*t3*t4
	else
		return 0
	endif
End